local version = "0.3";
local isDebug = false;

local taunts = {
	355,   -- Warrior: Taunt
	1161,  -- Warrior: Challenging Shout
	5209,  -- Druid: Challanging Roar
	6795,  -- Druid: Growl
	27270, -- VoidWalker: Torment Rank 7 
	31789, -- Paladin: Righteous Defense
	33700, -- Felguard: Anguish Rank 3
	33701, -- VoidWalker: Suffering Rank 6
	47984, -- VoidWalker: Torment Rank 8
	47989, -- VoidWalker: Suffering Rank 7
	47990, -- VoidWalker: Suffering Rank 8
	47993, -- Felguard: Anguish Rank 4
	56222, -- DK: Dark Command
	59671, -- Warlock: Challanging Howl
	62124, -- Paladin: Hand of Reckoning
}
local mocks = {	
	694,   -- Warrior: Mocking Blow
	49576, -- DK: Death Grip
	20736, -- Hunter: Distracting Shot
}
local knockbacks = {
	53227, -- Druid: Typhoon
	59159, -- Shaman: Thunderstorm
}
local buffRighteousFury = GetSpellInfo(25780)
local buffFrostPresence = GetSpellInfo(48263)

local PlayerClass;
local isTankMasteryEnabled = false;
local isMonitoringTaunts = false;

function TankMasteryFrame_OnLoad()	
	-- only register events if we can tank... 			
  PlayerClass = UnitClass("player"); 
  
  local isTank = isPossibleTank();  
  if (isTank > 0) then
	  		isTankMasteryEnabled = true
				TankMasteryFrame:UnregisterAllEvents();
	  		
				TankMasteryFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
				
				-- selective registering (optimization)
				if (isTank == 1) then
					TankMasteryFrame:RegisterEvent("UPDATE_SHAPESHIFT_FORM");
				elseif (isTank == 2) then
					TankMasteryFrame:RegisterEvent("UNIT_AURA");
				end				
				ChatFrame1:AddMessage("|cffffff00TankMastery Loaded (".. version ..")...");
				
				-- startup init
				updateTankingMode()
	else
		    isTankMasteryEnabled = false
				ChatFrame1:AddMessage("|cffffff00TankMastery NOT Loaded. We are not a tanking class.");
	end	
	
	TankMasteryFrame:Hide();	
end

function TankMasteryFrame_OnEvent(event, ...)
	-- deduce event type	
	if (event == "UPDATE_SHAPESHIFT_FORM") then
		updateTankingMode();
	elseif (event == "UNIT_AURA" and arg1 == "player" ) then
		updateTankingMode();
  elseif (event == "COMBAT_LOG_EVENT_UNFILTERED" and isMonitoringTaunts == true) then
		TankMasteryFrame_CombatLogEvent(event, ...);
  end
end

function updateTankingMode()
		if (isCurrentlyTanking() == true) then 
			debugPrint("Tanking...");
			isMonitoringTaunts = true;
		else
			debugPrint("NOT Tanking...");
			isMonitoringTaunts = false;
	  end
end

function TankMasteryFrame_CombatLogEvent(event, ...)
	local timestamp, ttype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellid, spellname, spellschool, b_or_d = select(1, ...);
	local tauntmsg = "";
	local isAOE = false;

	if (ttype == "SPELL_CAST_SUCCESS")
		and (UnitGUID("targettarget")) and (UnitGUID("target")) and (UnitGUID("targettarget") == UnitGUID("player")) and (UnitGUID("player") ~= srcGUID
		and (not CombatLog_Object_IsA(dstFlags, COMBATLOG_OBJECT_TYPE_PLAYER))) then
			if ((dstName == nil or dstName == "") and UnitName("target")) then			
				debugPrint("Non-targeted spell: ".. spellname);
				dstName = UnitName("target");
				isAOE = true;
			end

			-- taunts etc
			if (TableContainsValue(taunts, spellid)) then
				if (not isAOE) then -- only my target
					if (UnitGUID("target") == dstGUID) then
						-- ChatFrame1:AddMessage(srcName.." has taunted "..dstName.." of me using "..spellname..".");
						tauntmsg = srcName.." taunted "..dstName.." using "..spellname..".";
					end
				else
					tauntmsg = srcName.." taunted "..dstName.." using AOE "..spellname..".";
				end
			end

			-- mocks etc
			if (TableContainsValue(mocks, spellid)) then
				tauntmsg = srcName.." mocked "..dstName.." using "..spellname..".";
			end	

			-- knockback etc
			if (TableContainsValue(knockbacks, spellid)) then
				tauntmsg = srcName.." knocked-away "..dstName.." using "..spellname..".";
			end	

			if (tauntmsg ~= "") then		
				  if GetNumRaidMembers() ~= 0 then
					SendChatMessage(""..tauntmsg, "RAID")
				  elseif GetNumPartyMembers() ~= 0 then
					SendChatMessage(""..tauntmsg, "PARTY")
				  end
			end
	end
end

function debugPrint(msg)
	if (isDebug == true) then
		ChatFrame1:AddMessage("DEBUG : |cFF00FF00" .. msg);
	end	
end

function TableContainsValue(table, value)
        if (table and value) then
                for _, v in ipairs(table) do
                        if (v == value) then
                                return true;
                        end
                end
                return false;
        end
end

function isPossibleTank()
	if PlayerClass == "Druid" then 
		return 1;
	end
	if PlayerClass == "Warrior" then 
		return 1;
	end
	if PlayerClass == "Paladin" then 
		return 2;
	end
	if PlayerClass == "Death Knight" then 
		return 1;
	end 
	
	return 0;
end

function isCurrentlyTanking()
	-- Bear form
	if PlayerClass == "Druid" and GetShapeshiftForm(true) == 1 then 
		return true;
	end
	-- Defensive stance
	if PlayerClass == "Warrior" and GetShapeshiftForm(true) == 2 then 
		return true;
	end
	-- Righteous Fury 
	if PlayerClass == "Paladin" and UnitBuff("player", buffRighteousFury) then 
		return true;
	end
	-- Frost Presence  UnitBuff("player", buffFrostPresence) 
	if PlayerClass == "Death Knight" and GetShapeshiftForm(false) == 2 then 
		return true;
	end

	return false;
end